package eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.strike;

import com.google.common.reflect.TypeToken;
import com.google.gson.Gson;
import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.util.IteratorUtil;
import org.bukkit.Bukkit;
import org.bukkit.Location;

import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.lang.reflect.Type;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.time.Instant;
import java.util.*;

public class CordLeak {
    private static final Path saveFile = Paths.get(Main.instance().getDataFolder().getAbsolutePath(), "cordleak.json");
    private static final Map<UUID, LeakInfo> leaks = new HashMap<>();

    static {
        load();
    }

    private record LocationDto(String world, double x, double y, double z, float yaw, float pitch) {
        static LocationDto from(Location loc) {
            return new LocationDto(loc.getWorld().getName(), loc.getX(), loc.getY(), loc.getZ(), loc.getYaw(), loc.getPitch());
        }

        Location toLocation() {
            return new Location(Bukkit.getWorld(this.world), this.x, this.y, this.z, this.yaw, this.pitch);
        }
    }

    public record LeakInfo(Location location, Instant leakedAt) {}

    private record LeakInfoDto(LocationDto location, long leakedAt) {
        static LeakInfoDto from(LeakInfo info) {
            return new LeakInfoDto(LocationDto.from(info.location), info.leakedAt.toEpochMilli());
        }

        LeakInfo toLeakInfo() {
            return new LeakInfo(this.location.toLocation(), Instant.ofEpochMilli(this.leakedAt));
        }
    }

    public static void update() {
        IteratorUtil.onlinePlayers(player -> {
            UUID uuid = player.getUniqueId();
            Location loc = player.getLocation();

            leaks.compute(uuid, (key, existing) -> {
                // Falls bereits geleakt, nicht aktualisieren
                if (existing != null && !existing.leakedAt.equals(Instant.EPOCH)) {
                    return existing;
                }
                return new LeakInfo(loc, existing != null ? existing.leakedAt : Instant.EPOCH);
            });
        });
        Bukkit.getScheduler().runTaskAsynchronously(Main.instance(), CordLeak::save);
    }

    public static void markAsLeaked(UUID uuid) {
        LeakInfo existing = leaks.get(uuid);
        if (existing == null) return;
        leaks.put(uuid, new LeakInfo(existing.location, Instant.now()));
        Bukkit.getScheduler().runTaskAsynchronously(Main.instance(), CordLeak::save);
    }

    public static LeakInfo getLastKnownLocation(UUID uuid) {
        return leaks.get(uuid);
    }

    public static Map<UUID, LeakInfo> getAllLeaked() {
        Map<UUID, LeakInfo> result = new HashMap<>();
        Instant cutoff = Instant.now().minusSeconds(3 * 24 * 60 * 60); // 3 Tage in Sekunden

        for (Map.Entry<UUID, LeakInfo> entry : leaks.entrySet()) {
            LeakInfo info = entry.getValue();
            if (info.leakedAt.isAfter(cutoff)) {
                result.put(entry.getKey(), info);
            }
        }

        return result;
    }

    private static void load() {
        if (!Files.exists(saveFile)) return;
        try (Reader reader = Files.newBufferedReader(saveFile)) {
            Type type = new TypeToken<Map<String, LeakInfoDto>>() {}.getType();
            Map<String, LeakInfoDto> raw = new Gson().fromJson(reader, type);
            for (Map.Entry<String, LeakInfoDto> entry : raw.entrySet()) {
                leaks.put(UUID.fromString(entry.getKey()), entry.getValue().toLeakInfo());
            }
        } catch (IOException e) {
            Main.logger().warning("Failed to load CordLeak data: " + e.getMessage());
        }
    }

    private static void save() {
        try {
            Files.createDirectories(saveFile.getParent());
            Map<String, LeakInfoDto> raw = new HashMap<>();
            for (Map.Entry<UUID, LeakInfo> entry : leaks.entrySet()) {
                raw.put(entry.getKey().toString(), LeakInfoDto.from(entry.getValue()));
            }
            try (Writer writer = Files.newBufferedWriter(saveFile)) {
                new Gson().toJson(raw, writer);
            }
        } catch (IOException e) {
            Main.logger().warning("Failed to save CordLeak data: " + e.getMessage());
        }
    }
}
